@@ -25,34 +25,28 @@
 
 #include <kurl.h>
 
-#include <ktexteditor/cursor.h>
-
-#include "parser.h"
-#include "control.h"
-#include "dumpchain.h"
-
 #include <language/duchain/identifier.h>
-#include <language/duchain/dumpchain.h>
 #include <language/duchain/types/indexedtype.h>
-#include <language/editor/simplecursor.h>
+#include <language/editor/cursorinrevision.h>
+
+#include "testhelper.h"
 
 namespace KDevelop
 {
 class Declaration;
 class TopDUContext;
-class SimpleCursor;
 class ClassFunctionDeclaration;
 }
 
-class TestDUChain : public QObject
+class TestDUChain : public QObject, public Cpp::TestHelper
 {
   Q_OBJECT
 
 public:
   TestDUChain();
 
-  KDevelop::Declaration* findDeclaration(KDevelop::DUContext* context, const KDevelop::Identifier& id, const KDevelop::SimpleCursor& position = KDevelop::SimpleCursor::invalid());
-  KDevelop::Declaration* findDeclaration(KDevelop::DUContext* context, const KDevelop::QualifiedIdentifier& id, const KDevelop::SimpleCursor& position = KDevelop::SimpleCursor::invalid());
+  KDevelop::Declaration* findDeclaration(KDevelop::DUContext* context, const KDevelop::Identifier& id, const KDevelop::CursorInRevision& position = KDevelop::CursorInRevision::invalid());
+  KDevelop::Declaration* findDeclaration(KDevelop::DUContext* context, const KDevelop::QualifiedIdentifier& id, const KDevelop::CursorInRevision& position = KDevelop::CursorInRevision::invalid());
 
 
   //Move to slots again once it should be tested
@@ -88,6 +82,18 @@ private slots:
   void testSearchAcrossNamespace();
   void testSearchAcrossNamespace2();
   void testSearchAcrossNamespace3();
+  void testADL();
+  void testADLClassType();
+  void testADLFunctionType();
+  void testADLFunctionByName();
+  void testADLEnumerationType();
+  void testADLClassMembers();
+  void testADLMemberFunction();
+  void testADLMemberFunctionByName();
+  void testADLOperators();
+  void testADLNameAlias();
+  void testADLTemplateArguments();
+  void testADLTemplateTemplateArguments();
   void testTemplateEnums();
   void testIntegralTemplates();
   void testTypedef();
@@ -188,24 +194,8 @@ private:
   void assertNoMemberFunctionModifiers(KDevelop::ClassFunctionDeclaration* memberFun);
 
 public:
-  enum DumpArea {
-    DumpNone = 0,
-    DumpAST = 1,
-    DumpDUChain = 2,
-    DumpType = 4,
-    DumpAll = 7
-  };
-  Q_DECLARE_FLAGS(DumpAreas, DumpArea)
 
 private:
-  KDevelop::TopDUContext* parse(const QByteArray& unit, DumpAreas dump = static_cast<DumpAreas>(DumpAST | DumpDUChain | DumpType),
-                                 KDevelop::TopDUContext* update = 0, bool keepAst = false);
-
-  // Parser
-  Control control;
-  Cpp::DumpChain cppDumper;
-  KDevelop::DumpChain dumper;
-
   // Declaration - use chain
   KDevelop::Declaration* noDef;
   KUrl file1, file2;
