@@ -28,8 +28,10 @@
 
 #include <KDirWatch>
 
+#include <project/interfaces/iprojectfilemanager.h>
 #include <project/interfaces/ibuildsystemmanager.h>
 #include <language/interfaces/ilanguagesupport.h>
+#include <language/codegen/applychangeswidget.h>
 #include <interfaces/iplugin.h>
 #include <interfaces/idocumentationprovider.h>
 
@@ -37,6 +39,7 @@
 #include "icmakemanager.h"
 #include "cmakeprojectvisitor.h"
 
+struct CMakeProjectData;
 class QStandardItem;
 class QDir;
 class QObject;
@@ -87,10 +90,9 @@ public:
     virtual KDevelop::ProjectTargetItem* createTarget( const QString&, KDevelop::ProjectFolderItem* ) { return 0; }
     virtual bool addFileToTarget( KDevelop::ProjectFileItem*, KDevelop::ProjectTargetItem* );
 
-    virtual bool removeFolder( KDevelop::ProjectFolderItem* );
     virtual bool removeTarget( KDevelop::ProjectTargetItem* ) { return false; }
-    virtual bool removeFile( KDevelop::ProjectFileItem* );
-    virtual bool removeFileFromTarget( KDevelop::ProjectFileItem*, KDevelop::ProjectTargetItem* );
+    virtual bool removeFilesFromTargets( QList<KDevelop::TargetFilePair> );
+    virtual bool removeFilesAndFolders( QList<KDevelop::ProjectBaseItem*> items);
 
     virtual bool renameFile(KDevelop::ProjectFileItem*, const KUrl&);
     virtual bool renameFolder(KDevelop::ProjectFolderItem*, const KUrl&);
@@ -111,7 +113,7 @@ public:
     virtual QString name() const;
     virtual KDevelop::ParseJob *createParseJob(const KUrl &url);
     virtual KDevelop::ILanguage *language();
-    virtual const KDevelop::ICodeHighlighting* codeHighlighting() const;
+    virtual KDevelop::ICodeHighlighting* codeHighlighting() const;
     virtual QWidget* specialLanguageObjectNavigationWidget(const KUrl& url, const KDevelop::SimpleCursor& position);
 
 private slots:
@@ -131,19 +133,21 @@ private:
     
     QMutex m_reparsingMutex;
     QMutex m_busyProjectsMutex;
-    KDevelop::ReferencedTopDUContext initializeProject(KDevelop::IProject* project, const KUrl& baseUrl);
+    KDevelop::ReferencedTopDUContext initializeProject(KDevelop::IProject* project);
     
-    KDevelop::ReferencedTopDUContext includeScript(const QString& File, KDevelop::IProject * project,
+    KDevelop::ReferencedTopDUContext includeScript(const QString& file, KDevelop::IProject * project, const QString& currentDir,
                                                     KDevelop::ReferencedTopDUContext parent);
     
     static void setTargetFiles(KDevelop::ProjectTargetItem* target, const KUrl::List& files);
     void reloadFiles(KDevelop::ProjectFolderItem* item);
 
-    QMap<KDevelop::IProject*, QStringList> m_modulePathPerProject;
-    QMap<KDevelop::IProject*, VariableMap> m_varsPerProject;
-    QMap<KDevelop::IProject*, MacroMap> m_macrosPerProject;
+    bool changesWidgetAddTargetFileRemovals(const QList<KDevelop::TargetFilePair> &targetFiles, KDevelop::ApplyChangesWidget* changesWidget);
+    bool changesWidgetAddCMakeFolderRemovals(const QList<CMakeFolderItem*> &folders, KDevelop::ApplyChangesWidget* changesWidget);
+    QList<CMakeFolderItem*> getCMakeFoldersWithin(const QList<KDevelop::ProjectBaseItem*> &items) const;
+    QList<KDevelop::TargetFilePair> getTargetFilesWithin(const QList<KDevelop::ProjectBaseItem*> &items) const;
+
+    QMap<KDevelop::IProject*, CMakeProjectData> m_projectsData;
     QMap<KDevelop::IProject*, KDirWatch*> m_watchers;
-    QMap<KDevelop::IProject*, CacheValues> m_projectCache;
     QMap<KUrl, CMakeFolderItem*> m_pending;
     
     QSet<KDevelop::ProjectFolderItem*> m_busyProjects;
